#include <iostream>
#include <cuda_runtime.h>

__global__ void processArray(int *arr, int N) {
    // Get the index of the thread
    int idx = threadIdx.x;

    // Ensure the index is within bounds
    if (idx < N) {
        arr[idx] *= 2; // Multiply each element by 2
    }
}

int main() {
    const int N = 1000; // Size of the array
    int *arr;

    // Allocate managed memory (accessible by both CPU and GPU)
    cudaError_t err = cudaMallocManaged(&arr, N * sizeof(int));

    if (err != cudaSuccess) {
        std::cout << "Error: " << cudaGetErrorString(err) << std::endl;
        return -1;
    }

    // Initialize the array
    for (int i = 0; i < N; ++i) {
        arr[i] = i;
    }

    // Launch kernel with a single block of threads
    processArray<<<1, N>>>(arr, N);

    // Wait for GPU to finish before proceeding
    cudaDeviceSynchronize();

    // Print first 10 elements to verify
    for (int i = 0; i < 10; ++i) {
        std::cout << arr[i] << " ";
    }
    std::cout << std::endl;

    // Free the managed memory
    cudaFree(arr);

    return 0;
}

